// Searches files in the Shareware BONANZA disk set.

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <dos.h>
#include <malloc.h>
#include <string.h>
#include "searchcd.h"

#define LISTS "\\LISTS"

struct sect_struct {
    char *catalog;
    char *name;
    char *desc;
} ;

struct sect_struct sect_list[50];
int no_secs=0;
struct sect_struct a_sect[50];
int no_entries=0;

unsigned original_drive;
char cwd[200];


void reset() {
    unsigned dummy;

    _dos_setdrive(original_drive,&dummy);
    chdir(cwd);
}

void ioerror(char *text) {
    reset();
    printf("IO error.  %s\n",text);
    exit(1);
}

void setcddir() {
    unsigned dummy;

    _dos_setdrive(CONTROL.DRIVE[0]-'@',&dummy);
    if (chdir(LISTS)==-1) ioerror("Cannot access CD-ROM.  Try SETUP.");
}

char equal(char *given, char *trial) {
     char passed=1;

     if (trial[0]=='*') return 1;
     if (strcmp(given,trial)==0) return 1;

     while (passed) {
// if we come to a [], just assume the rest of the string is optional.
          if (*trial=='[')
              return 1;
          if (*given==0 && *trial==0) return 1;
          if (*given==0 || *trial==0) return 0;
          passed=(*trial == *given);
          trial++;
          given++;
     }
     return 0;
}

char parms(int argc, char *argv[], char *start, ... ) {
     char *next;
     va_list arg_ptr;
     int ptr=1;

     if (argc==1) return 0;

     va_start(arg_ptr,start);
     next=start;

     while (next && argc>1) {
         strupr(argv[ptr]);
         strupr(next);

         if (!equal(argv[ptr],next))
             return 0;
         next=va_arg(arg_ptr,char *);
         ptr++;
         argc--;
     } ;
     va_end(arg_ptr);

     if (argc!=1)
          return 0;
     return 1;
}

void read_cat_line(FILE *cat, char *dest) {
    dest[0]=0;
    fgets(dest,100,cat);
    if (dest[0]==0 || dest[0]=='\n') {
       dest[0]=0;
       return;
    }
    while (dest[0]==';') {
        fgets(dest,100,cat);
        if (dest[0]==0 || dest[0]=='\n') {
            dest[0]=0;
            return;
        }
    }
    dest[strlen(dest)-1]=0;
}

void get_sections() {
    struct find_t dirs;
    FILE *catalog;
    char cat_line[100];
    char *desc;
    int list_ptr=0;

    setcddir();

// If sections have been read, do not re-read.
    if (no_secs) return;

    if (_dos_findfirst("*.CAT",_A_NORMAL,&dirs)) ioerror("Cannot get directory.  Try SETUP.");

    do {
         catalog=fopen(dirs.name,"r");
         if (catalog==NULL) ioerror("Cannot open control file.  Check FILES in CONFIG.SYS.");
         read_cat_line(catalog,cat_line);
         desc=strstr(cat_line," ");
         desc++;
         desc[-1]=0;
         sect_list[list_ptr].catalog=malloc(strlen(dirs.name)+1);
         sect_list[list_ptr].name=malloc(strlen(cat_line)+1);
         sect_list[list_ptr].desc=malloc(strlen(desc)+1);
         strcpy(sect_list[list_ptr].catalog,dirs.name);
         strcpy(sect_list[list_ptr].name,cat_line);
         strcpy(sect_list[list_ptr].desc,desc);
         fclose(catalog);
         list_ptr++;
    } while (!_dos_findnext(&dirs));
    no_secs=list_ptr;
}

void list_sections() {
     int i;

     get_sections();

     printf("List of sections on current CD:\n\n");

     for (i=0;i<no_secs;i++)
        printf("   %s   %s\n",sect_list[i].name,sect_list[i].desc);
}

void get_section(char *section) {
    int i=0,line_no=0;
    FILE *catalog;
    char cat_line[100];
    char *desc;

    get_sections();
    strupr(section);

    while (strcmp(sect_list[i].name,section)!=0 && i<no_secs)
         i++;
    if (i>=no_secs) {
        printf("Section not found on disk.\n");
        exit(1);
    }

    catalog=fopen(sect_list[i].catalog,"r");
    if (!catalog) ioerror("Cannot open control file.  Check FILES in CONFIG.SYS.");

    read_cat_line(catalog,cat_line);

    while (1) {
        read_cat_line(catalog,cat_line);
        if (cat_line[0]==0) {
            no_entries=line_no;
            fclose(catalog);
            return;
        }

        desc=strstr(cat_line," ");
        desc++;
        desc[-1]=0;

        a_sect[line_no].name=malloc(strlen(cat_line)+1);
        a_sect[line_no].desc=malloc(strlen(desc)+1);
        strcpy(a_sect[line_no].name,cat_line);
        strcpy(a_sect[line_no].desc,desc);

        line_no++;
    }
}

void list_section(char *section) {
     int i;

     setcddir();
     get_section(section);

     printf("List of entries in section %s:\n\n",section);

     for (i=0;i<no_entries;i++)
        printf("   %s   %s\n",a_sect[i].name,a_sect[i].desc);
}

void search_list(char *section, char *entry, char *text) {
    char pathname[200];
    FILE *list;
    char line[100],lineupr[100];
    char *restofline;

    strupr(text);

    strcpy(pathname,CONTROL.DRIVE);
    strcat(pathname,LISTS);
    strcat(pathname,"\\");
    strcat(pathname,section);
    strcat(pathname,"\\");
    strcat(pathname,entry);
    strcat(pathname,".LIS");
    list=fopen(pathname,"r");

    if (!list) ioerror("Cannot open listing file.  Check parameters.\n");

    while (!feof(list)) {
        fgets(line,sizeof(line)-1,list);
        strcpy(lineupr,line);
        strupr(lineupr);
        if (strstr(lineupr,text)!=NULL) {
            strcpy(pathname,CONTROL.DRIVE);
            strcat(pathname,"\\");
            strcat(pathname,section);
            strcat(pathname,"\\");
            strcat(pathname,line);
            restofline=strstr(pathname," ");
            restofline[0]=0;
            do {
               restofline++;
            } while (restofline[0]==' ');
            printf("%s:\n     %s",pathname,restofline);
        }
    }
    fclose(list);
}

void search_entry(char *section, char *entry, char *searchtext) {
    printf("Searching for %s in section %s:%s:\n\n",searchtext,section,entry);

    search_list(section,entry,searchtext);
}

void search_section(char *section, char *searchtext) {
    int i;

    get_sections();
    get_section(section);

    for (i=0;i<no_entries;i++) {
        search_entry(section,a_sect[i].name,searchtext);
        printf("\n");
    }
}

void search_disk(char *searchtext) {
    int i;

    printf("Searching for %s in whole disk:\n\n",searchtext);

    get_sections();

    for (i=0;i<no_secs;i++)
        search_section(sect_list[i].name,searchtext);
}

char *form_text_entry(int entryno, char *argv[]) {
     char text_entry[256];
     char *ret;

     strcpy(text_entry,argv[entryno]);

     if (strstr(&text_entry[1],"\"")!=NULL) {
         ret=malloc(strlen(text_entry)+1);
         strcpy(ret,text_entry);
         return ret;
     }
     do {
         entryno++;
         strcat(text_entry,argv[entryno]);
         if (strstr(&text_entry[1],"\"")!=NULL) {
              ret=malloc(strlen(text_entry)+1);
              strcpy(ret,text_entry);
              return ret;
         }
     } while (1);
}

void main(int argc,  char *argv[]) {
     char *search_text;

     _dos_getdrive(&original_drive);
     getcwd(cwd,sizeof(cwd)-1);
     if (parms(argc,argv,"l[ist]",NULL))
          list_sections();
     else if (parms(argc,argv,"l[ist]","*",NULL))
          list_section(argv[2]);
     else if (parms(argc,argv,"s[earch]","*",NULL))
          search_disk(argv[2]);
     else if (parms(argc,argv,"s[earch]","*","*",NULL))
          search_section(argv[2],argv[3]);
     else if (parms(argc,argv,"s[earch]","*","*","*",NULL))
          search_entry(argv[2],argv[3],argv[4]);
     else {
          printf("SEARHCD, Copyright (c) 1992 by Joe Tamburino.\n");
          printf("    (508) 692-7756        (home)\n");
          printf("    (617) 890-1100, X7871 (work)\n");
          printf("    7 Christopher Rd.\n");
          printf("    Westford, MA  01886\n\n");
          printf("Usage:\n\n");
          printf("    SEARCHCD l[ist] ...................... List all sections on CD\n");
          printf("    SEARCHCD l[ist] section .............. List entries of specific section\n");
          printf("    SEARCHCD s[earch] \"text\" ............. Search entire disk for text\n");
          printf("    SEARCHCD s[earch] section \"text\" ..... Search entire section for text\n");
          printf("    SEARCHCD s[earch] sect entry \"text\" .. Search particular section:entry\n");
          printf("\n");
     }
     reset();
}
